@@ -27,8 +27,9 @@ use std::fs;
 
 
 lazy_static! {
-    static ref CREDENTIALS_REGEX: Regex = Regex::new("[^:]+(?::[^:]+)?").unwrap();
-    static ref PATH_CREDENTIALS_REGEX: Regex = Regex::new("(.+)=([^:]+(?::[^:]+)?)?").unwrap();
+    static ref CREDENTIALS_REGEX: Regex = Regex::new("^[^:]+(?::[^:]+)?$").unwrap();
+    static ref PATH_CREDENTIALS_REGEX: Regex = Regex::new("^(.+)=([^:]+(?::[^:]+)?)?$").unwrap();
+    static ref HEADER_REGEX: Regex = Regex::new("^([^:]+):[[:space:]]*(.+)$").unwrap();
 }
 
 
@@ -76,6 +77,8 @@ pub struct Options {
     /// The temp directory to write to before copying to hosted directory and to store encoded FS responses.
     /// Default: `"$TEMP/http-[FULL_PATH_TO_HOSTED_DIR]"`
     pub temp_directory: (String, PathBuf),
+    /// Whether to generate directory listings at all. Default: true
+    pub generate_listings: bool,
     /// Whether to check for index files in served directories before serving a listing. Default: true
     pub check_indices: bool,
     /// Whether to allow requests to `/file` to return `/file.{INDEX_EXTENSIONS`. Default: false
@@ -104,10 +107,14 @@ pub struct Options {
     pub generate_path_auth: BTreeSet<String>,
     /// Header names and who we trust them from in `HEADER-NAME:CIDR` format
     pub proxies: BTreeMap<IpCidr, String>,
+    /// Header names and who we trust them from in `HEADER-NAME:CIDR` format
+    pub proxy_redirs: BTreeMap<IpCidr, String>,
     /// Extension -> MIME type mapping overrides; empty string for no extension
     pub mime_type_overrides: BTreeMap<String, Mime>,
     /// Max amount of data per second each request is allowed to return. Default: `None`
     pub request_bandwidth: Option<NonZeroU64>,
+    /// Additional headers to add to every response
+    pub additional_headers: Vec<(String, Vec<u8>)>,
 }
 
 impl Options {
@@ -128,9 +135,10 @@ impl Options {
             .arg(Arg::from_usage("-r --sandbox-symlinks 'Restrict/sandbox where symlinks lead to only the direct descendants of the hosted directory. \
                                   Default: false'"))
             .arg(Arg::from_usage("-w --allow-write 'Allow for write operations. Default: false'"))
-            .arg(Arg::from_usage("-i --no-indices 'Always generate dir listings even if index files are available. Default: false'"))
+            .arg(Arg::from_usage("-l --no-listings 'Never generate dir listings. Default: false'"))
+            .arg(Arg::from_usage("-i --no-indices 'Do not automatically use index files. Default: false'"))
             .arg(Arg::from_usage("-e --no-encode 'Do not encode filesystem files. Default: false'"))
-            .arg(Arg::from_usage("-x --strip-extensions 'Allow stripping index extentions from served paths. Default: false'"))
+            .arg(Arg::from_usage("-x --strip-extensions 'Allow stripping index extensions from served paths. Default: false'"))
             .arg(Arg::from_usage("-q --quiet... 'Suppress increasing amounts of output'"))
             .arg(Arg::from_usage("-c --no-colour 'Don't colourise the log output'"))
             .arg(Arg::from_usage("-d --webdav 'Handle WebDAV requests. Default: false'"))
@@ -140,14 +148,23 @@ impl Options {
             .arg(Arg::from_usage("--auth [USERNAME[:PASSWORD]] 'Data for global authentication'").validator(Options::credentials_validator))
             .arg(Arg::from_usage("--gen-auth 'Generate a one-off username:password set for global authentication'").conflicts_with("auth"))
             .arg(Arg::from_usage("--path-auth [PATH=[USERNAME[:PASSWORD]]]... 'Data for authentication under PATH'")
+                .use_delimiter(false)
                 .validator(Options::path_credentials_validator))
-            .arg(Arg::from_usage("--gen-path-auth [PATH]... 'Generate a one-off username:password set for authentication under PATH'"))
+            .arg(Arg::from_usage("--gen-path-auth [PATH]... 'Generate a one-off username:password set for authentication under PATH'").use_delimiter(false))
             .arg(Arg::from_usage("--proxy [HEADER-NAME:CIDR]... 'Treat HEADER-NAME as proxy forwarded-for header when request comes from CIDR'")
+                .use_delimiter(false)
+                .validator(|s| Options::proxy_parse(s.into()).map(|_| ())))
+            .arg(Arg::from_usage("--proxy-redir [HEADER-NAME:CIDR]... 'Treat HEADER-NAME as proxy X-Original-URL header for redirects when request comes from CIDR'")
+                .use_delimiter(false)
                 .validator(|s| Options::proxy_parse(s.into()).map(|_| ())))
             .arg(Arg::from_usage("-m --mime-type [EXTENSION:MIME-TYPE]... 'Always return MIME-TYPE for files with EXTENSION'")
+                .use_delimiter(false)
                 .validator(|s| Options::mime_type_override_parse(s.into()).map(|_| ())))
             .arg(Arg::from_usage("--request-bandwidth [BYTES] 'Limit each request to returning BYTES per second, or 0 for unlimited. Default: 0'")
                 .validator(|s| Options::bandwidth_parse(s.into()).map(|_| ())))
+            .arg(Arg::from_usage("-H --header [NAME: VALUE]... 'Headers to add to every response'")
+                .use_delimiter(false)
+                .validator(|s| Options::header_parse(&s).map(|_| ())))
             .get_matches();
 
         let dir = matches.value_of("DIR").unwrap_or(".");
@@ -210,6 +227,7 @@ impl Options {
                          suffix),
                  temp_pb.join(suffix))
             },
+            generate_listings: !matches.is_present("no-listings"),
             check_indices: !matches.is_present("no-indices"),
             strip_extensions: matches.is_present("strip-extensions"),
             allow_writes: matches.is_present("allow-write"),
@@ -222,6 +240,7 @@ impl Options {
             path_auth_data: path_auth_data,
             generate_path_auth: generate_path_auth,
             proxies: matches.values_of("proxy").unwrap_or_default().map(Cow::from).map(Options::proxy_parse).map(Result::unwrap).collect(),
+            proxy_redirs: matches.values_of("proxy-redir").unwrap_or_default().map(Cow::from).map(Options::proxy_parse).map(Result::unwrap).collect(),
             mime_type_overrides: matches.values_of("mime-type")
                 .unwrap_or_default()
                 .map(Cow::from)
@@ -229,6 +248,11 @@ impl Options {
                 .map(Result::unwrap)
                 .collect(),
             request_bandwidth: matches.value_of("request-bandwidth").map(Cow::from).map(Options::bandwidth_parse).map(Result::unwrap).unwrap_or_default(),
+            additional_headers: matches.values_of("header")
+                .unwrap_or_default()
+                .map(Options::header_parse)
+                .map(Result::unwrap)
+                .collect(),
         }
     }
 
@@ -371,4 +395,8 @@ impl Options {
             }
         }
     }
+
+    fn header_parse(s: &str) -> Result<(String, Vec<u8>), String> {
+        HEADER_REGEX.captures(s).map(|hdr| (hdr[1].to_string(), hdr[2].as_bytes().to_vec())).ok_or_else(|| format!("\"{}\" invalid format", s))
+    }
 }
